package be.iminds.ilabt.jfed.testing.tests.highlevel;

import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.jobs.report.JobReport;
import be.iminds.ilabt.jfed.highlevel.jobs.report.JobStateReport;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.legacy.TargetAuthority;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedCorePreferences;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.testing.base.ApiTest;
import be.iminds.ilabt.jfed.testing.base.ApiTestConfig;
import be.iminds.ilabt.jfed.testing.base.ApiTestResult;
import be.iminds.ilabt.jfed.testing.shared.Proxy;
import be.iminds.ilabt.jfed.testing.tests.highlevel.config.ESpecTestConfig;
import com.google.inject.Injector;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.util.Collection;
import java.util.Collections;

/**
 * AbstractHLTest:
 *
 * For now, this just adds the addEmbeddedResult method
 *
 * TODO: add all shared code of ESpecTest and GuiLogicTest here
 */
public abstract class AbstractHLTest<SpecificApiTestConfig extends ApiTestConfig>
        extends ApiTest<SpecificApiTestConfig> {

    public AbstractHLTest(@Nonnull Logger logger,
                       @Nullable TargetAuthority testedAuthority,
                       @Nullable GeniUserProvider geniUserProvider,
                       @Nonnull SpecificApiTestConfig testConfig) {
        super(logger, testedAuthority, geniUserProvider,
                testConfig);
    }

    protected void addEmbeddedResult(Collection<JobReport> jobReports) {
        for (JobReport jobReport: jobReports) {
            addEmbeddedResult(jobReport);
        }
    }
    protected void addEmbeddedResult(JobReport jobReport) {
        if (!jobReport.getStatelessLogLines().isEmpty()) {
            long durationMs =
                    jobReport.getEndTime() == null || jobReport.getStartTime() == null ?
                            0L :
                            jobReport.getEndTime().toEpochMilli() - jobReport.getStartTime().toEpochMilli();
            LogOutput.TestResultState jobState = LogOutput.TestResultState.SUCCESS; //TODO figure out job state
            ApiTestResult.ApiTestMethodResult res = new ApiTestResult.ApiTestMethodResult(
                    jobReport.getName()+" general logging",
                    jobReport.getName()+" general logging",
                    Collections.emptyList(), //soft deps
                    Collections.emptyList(), //hard deps
                    Collections.emptyList(), //embedded
                    jobReport.getStatelessLogLines(), //logLines
                    Collections.emptyList(), //apiCallDetails
                    null, //exception
                    jobState,
                    durationMs,
                    jobReport.getStartTime() == null ? 0L : jobReport.getStartTime().toEpochMilli(),
                    jobReport.getEndTime() == null ? 0L : jobReport.getEndTime().toEpochMilli()
            );
            currentTestResult.addEmbedded(res);
        }
        for (JobStateReport jobStateReport: jobReport.getJobStateReports()) {
            addEmbeddedResult(jobStateReport);
        }
    }

    protected void addEmbeddedResult(@Nonnull JobStateReport jobStateReport) {
        long durationMs =
                jobStateReport.getEndTime() == null || jobStateReport.getStartTime() == null ?
                        0L :
                        jobStateReport.getEndTime().toEpochMilli() - jobStateReport.getStartTime().toEpochMilli();
        LogOutput.TestResultState state = jobStateReport.getFinalStatus() == null ?
                LogOutput.TestResultState.FAILED : //null must mean something has gone wrong
                jobStateReport.getFinalStatus().toTestResultState();
        ApiTestResult.ApiTestMethodResult res = new ApiTestResult.ApiTestMethodResult(
                jobStateReport.getStateName(),
                jobStateReport.getStateName(),
                Collections.emptyList(), //soft deps
                Collections.emptyList(), //hard deps
                Collections.emptyList(), //embedded
                jobStateReport.getLogLines(), //logLines
                Collections.emptyList(), //apiCallDetails
                null, //exception
                state,
                durationMs,
                jobStateReport.getStartTime() == null ? 0L : jobStateReport.getStartTime().toEpochMilli(),
                jobStateReport.getEndTime() == null ? 0L : jobStateReport.getEndTime().toEpochMilli()
        );
        currentTestResult.addEmbedded(res);
    }
}
